package parkomat.ui.controller;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Locale;
import java.util.Map;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import parkomat.model.MyTicketIssuer;
import parkomat.model.TariffaValida;
import parkomat.model.Ticket;
import parkomat.model.TicketIssuer;

public class MyController implements Controller {
	private Map<String,TariffaValida> mappaTariffe;
	private TicketIssuer tickerIssuer;

	public MyController(Map<String, TariffaValida> mappaTariffeValide) {
		this.mappaTariffe=mappaTariffeValide;
		tickerIssuer = new MyTicketIssuer(mappaTariffeValide);
	}

	@Override
	public ObservableList<String> getNomiTariffe() {
		return FXCollections.observableArrayList(mappaTariffe.keySet());
	}

	@Override
	public Ticket emitTicket(String nomeTariffa, LocalDate dataInizio, String oraInizio, LocalDate dataFine, String oraFine) {
		DateTimeFormatter formatter = DateTimeFormatter.ofLocalizedTime(FormatStyle.SHORT).withLocale(Locale.ITALY);
		LocalDateTime inizioSosta = LocalDateTime.of(dataInizio, LocalTime.parse(oraInizio, formatter));
		LocalDateTime fineSosta = LocalDateTime.of(dataFine, LocalTime.parse(oraFine, formatter));
		if (inizioSosta.isAfter(fineSosta)) {
			Controller.alert("Errore", "Impossibile tornare indietro nel tempo", "La data di fine sosta è precedente quella di inizio sosta");
			return null;
		}		
		return tickerIssuer.emitTicket(nomeTariffa, inizioSosta, fineSosta);
	}
	
}
