package parkomat.tests.model;

import static org.junit.Assert.*;

import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import parkomat.model.*;
import parkomat.model.PagamentoAttivo;

public class TariffaValidaTest
{

	private List<FasciaTariffaria> fasceFeriali;
	private Map<DayOfWeek, List<FasciaTariffaria>> mappaFasce, mappaFasceSenzaGiovedi, mappaFasceConMartediBucato;
	
	@Before
	public void setUp() {
		/*
		Tariffa H1
		costo 0.50, franchigia 0, minimo 60
		Lunedì,	  8-20
		Martedì,  8-20
		Mercoledì,8-20
		Giovedì,  8-20
		Venerdì,  8-20
		Sabato,	  8-20
		NON completata AUTOMATICAMENTE sulle fasce gratuite
		 */
		fasceFeriali = new ArrayList<>();
		fasceFeriali.add(new FasciaTariffaria(LocalTime.of(8,0), LocalTime.of(20,00),PagamentoAttivo.A_PAGAMENTO));
		 
		mappaFasce = new HashMap<>();
		mappaFasce.put(DayOfWeek.MONDAY, fasceFeriali);
		mappaFasce.put(DayOfWeek.TUESDAY, fasceFeriali);
		mappaFasce.put(DayOfWeek.WEDNESDAY, fasceFeriali);
		mappaFasce.put(DayOfWeek.THURSDAY, fasceFeriali);
		mappaFasce.put(DayOfWeek.FRIDAY, fasceFeriali);
		mappaFasce.put(DayOfWeek.SATURDAY, fasceFeriali);
		mappaFasce.put(DayOfWeek.SUNDAY, fasceFeriali);		

		mappaFasceSenzaGiovedi = new HashMap<>();
		mappaFasceSenzaGiovedi.put(DayOfWeek.MONDAY, fasceFeriali);
		mappaFasceSenzaGiovedi.put(DayOfWeek.TUESDAY, fasceFeriali);
		mappaFasceSenzaGiovedi.put(DayOfWeek.WEDNESDAY, fasceFeriali);
		// mappaFasceSenzaGiovedi.put(DayOfWeek.THURSDAY, fasceFeriali);
		mappaFasceSenzaGiovedi.put(DayOfWeek.FRIDAY, fasceFeriali);
		mappaFasceSenzaGiovedi.put(DayOfWeek.SATURDAY, fasceFeriali);
		mappaFasceSenzaGiovedi.put(DayOfWeek.SUNDAY, fasceFeriali);		

		mappaFasceConMartediBucato = new HashMap<>();
		mappaFasceConMartediBucato.put(DayOfWeek.MONDAY, fasceFeriali);
		mappaFasceConMartediBucato.put(DayOfWeek.TUESDAY, List.of(new FasciaTariffaria(LocalTime.of(8,0), LocalTime.of(20,0),PagamentoAttivo.A_PAGAMENTO)));
		mappaFasceConMartediBucato.put(DayOfWeek.WEDNESDAY, fasceFeriali);
		mappaFasceConMartediBucato.put(DayOfWeek.THURSDAY, fasceFeriali);
		mappaFasceConMartediBucato.put(DayOfWeek.FRIDAY, fasceFeriali);
		mappaFasceConMartediBucato.put(DayOfWeek.SATURDAY, fasceFeriali);
		mappaFasceConMartediBucato.put(DayOfWeek.SUNDAY, fasceFeriali);				
	}
	

	@Test
	public void testInteraSettimanaTariffaCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasce);
		assertEquals("H1", t.getNomeTariffa());
		assertFalse(t.isValid());
		TariffaValida tv = new TariffaValida(t);
		assertTrue(tv.isValid());
	}

	@Test
	public void testSoloLunediTariffaCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasce);
		assertEquals("H1", t.getNomeTariffa());
		assertFalse(t.isValid(DayOfWeek.MONDAY));
		TariffaValida tv = new TariffaValida(t);
		assertTrue(tv.isValid()); 
		assertTrue(t.isValid(DayOfWeek.MONDAY));
	}

	@Test
	public void testTuttiGiorniTariffaCostruitaConUnaFasciaAllaVolta()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60);
		for (DayOfWeek day : DayOfWeek.values()) {
			for (FasciaTariffaria f: fasceFeriali) t.addFascia(f, day);
			assertFalse(t.isValid(day));
		}
		assertEquals("H1", t.getNomeTariffa());
		TariffaValida tv = new TariffaValida(t);
		assertTrue(tv.isValid());
	}

	@Test
	public void testTuttiGiorniTariffaCostruitaConUnaFasciaAllaVoltaARitroso()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60);
		FasciaTariffaria f = null;
		for (DayOfWeek day : DayOfWeek.values()) {
			for (ListIterator<FasciaTariffaria> it = fasceFeriali.listIterator(fasceFeriali.size()); it.hasPrevious(); ) {
				f = it.previous();
				t.addFascia(f, day);
			}
			assertFalse(t.isValid(day));
		}
		assertEquals("H1", t.getNomeTariffa());
		TariffaValida tv = new TariffaValida(t);
		assertTrue(tv.isValid());	}
	
	// casi che devono fallire
	
	@Test // la lista del giovedì è null
	public void testInteraSettimanaTariffaSenzaGiovediCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasceSenzaGiovedi);
		assertEquals("H1", t.getNomeTariffa());
		assertFalse(t.isValid());
		TariffaValida tv = new TariffaValida(t);
		assertTrue(tv.isValid());
	}

	@Test // la lista del martedì non è null, ma non è valida perché non copre le 24 ore
	public void testInteraSettimanaTariffaBucataMartediCostruitaConTutteLeFasceInsieme()
	{
		Tariffa t = new Tariffa("H1", 0.50, 0, 60, mappaFasceConMartediBucato);
		assertEquals("H1", t.getNomeTariffa());
		assertFalse(t.isValid());
	}

}
