package parkomat.tests.persistence;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.StringReader;
import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.Map;

import org.junit.Test;

import parkomat.model.*;
import parkomat.persistence.*;

public class MyTariffeReaderTests
{

	@Test
	public void testLeggiTariffe() throws IOException, BadFileFormatException
	{
		String toRead =
				"Tariffa C\r\n" + 
				"costo 0.50, franchigia 60, minimo 60\r\n" + 
				"Lunedì,	  8-13\r\n" + 
				"Lunedì,	 15-20\r\n" + 
				"Martedì,  8-13\r\n" + 
				"Martedì, 15-20\r\n" + 
				"Mercoledì,8-13\r\n" + 
				"Giovedì,  8-13\r\n" + 
				"Giovedì, 15-20\r\n" + 
				"Venerdì,  8-13\r\n" + 
				"Venerdì, 15-20\r\n" + 
				"Sabato,	  8-13\r\n" + 
				"Sabato,	 15-20\r\n" + 
				"Fine tariffa\r\n" + 
				"Tariffa H1\r\n" + 
				"costo 0.50, franchigia 0, minimo 60\r\n" + 
				"Lunedì,	  8-20\r\n" + 
				"Martedì,  8-20\r\n" + 
				"Mercoledì,8-20\r\n" + 
				"Giovedì,  8-20\r\n" + 
				"Venerdì,  8-20\r\n" + 
				"Sabato,	  8-20\r\n" + 
				"Fine tariffa\r\n" + 
				"Tariffa SolemareInvernale\r\n" + 
				"costo 0.50, franchigia 0, minimo 0\r\n" + 
				"Lunedì,	  8-18\r\n" + 
				"Martedì,  8-18\r\n" + 
				"Mercoledì,8-12\r\n" + 
				"Mercoledì,15-18\r\n" + 
				"Giovedì,  8-18\r\n" + 
				"Venerdì,  8-18\r\n" + 
				"Fine tariffa\r\n" + 
				"Tariffa SolemareEstiva\r\n" + 
				"costo 1.50, franchigia 0, minimo 0\r\n" + 
				"Lunedì,	  8-21\r\n" + 
				"Martedì,  8-21\r\n" + 
				"Mercoledì,8-21\r\n" + 
				"Giovedì,  8-21\r\n" + 
				"Venerdì,  8-21\r\n" + 
				"Sabato,	  8-21\r\n" + 
				"Domenica, 8-21\r\n" + 
				"Fine tariffa\r\n" + 
				"";
		
		StringReader reader = new StringReader(toRead);
		MyTariffeReader iscrittiReader = new MyTariffeReader();
		Map<String,Tariffa> mappaTariffe = iscrittiReader.leggiTariffe(reader);
		
		assertEquals(4, mappaTariffe.size());
		//
		Tariffa h1 = mappaTariffe.get("H1");
		assertTrue(0==h1.getFasce(DayOfWeek.SUNDAY).size());
		assertTrue(1==h1.getFasce(DayOfWeek.FRIDAY).size());
		assertEquals(LocalTime.of(20,0), h1.getFasce(DayOfWeek.FRIDAY).get(0).getOraFinoA());
		assertEquals(LocalTime.of(8,0),  h1.getFasce(DayOfWeek.FRIDAY).get(0).getOraDa());
		//
		Tariffa smi = mappaTariffe.get("SolemareInvernale");
		assertTrue(0==smi.getFasce(DayOfWeek.SUNDAY).size());
		assertTrue(1==smi.getFasce(DayOfWeek.FRIDAY).size());
		assertTrue(2==smi.getFasce(DayOfWeek.WEDNESDAY).size());
		assertEquals(LocalTime.of(8,0),  smi.getFasce(DayOfWeek.WEDNESDAY).get(0).getOraDa());
		assertEquals(LocalTime.of(12,0), smi.getFasce(DayOfWeek.WEDNESDAY).get(0).getOraFinoA());
		assertEquals(LocalTime.of(15,0), smi.getFasce(DayOfWeek.WEDNESDAY).get(1).getOraDa());
		assertEquals(LocalTime.of(18,0), smi.getFasce(DayOfWeek.WEDNESDAY).get(1).getOraFinoA());
		//
		Tariffa sm = mappaTariffe.get("SolemareEstiva");
		assertTrue(1==sm.getFasce(DayOfWeek.SUNDAY).size());
		assertTrue(1==sm.getFasce(DayOfWeek.FRIDAY).size());
		assertEquals(LocalTime.of(21,0), sm.getFasce(DayOfWeek.SUNDAY).get(0).getOraFinoA());
		assertEquals(LocalTime.of(8,0),  sm.getFasce(DayOfWeek.SUNDAY).get(0).getOraDa());

	}	
}
